/* Copyright 1999 by Herve Regad-Pellagru, E-mail: herve.regadpellagru@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#include <x10_api.h>

#include "proto.h"
#include "common.h"


extern int sock,listen_mode;
extern struct x10_status_t *x10_status;


/* We accept an incoming connection and returns its handle or return 0 if
   no connection occur */
static int do_accept(int acc_sock)
{
  int len,ret;
  static char string[MAXBUF];
  struct sockaddr from;
  struct hostent *peer;

  memset((char *)&from,0,sizeof(from));
  len=sizeof(from);

  /* always set listen_mode to 0 by default */
  listen_mode=0;
  ret=accept(acc_sock,&from,&len);

  /* As acc_sock is in non-blocking mode, we may be returned -1 if no connection
     occur. Otherwise, get who is ringing and log it */
  if(ret != -1)
    {
      if((peer = gethostbyaddr(from.sa_data+2,4,AF_INET)))
	  sprintf(string,"Connection from %s (%d.%d.%d.%d)\n",peer->h_name,
		  from.sa_data[2],from.sa_data[3],
		  from.sa_data[4],from.sa_data[5]);
      else
	sprintf(string,"Connection from unknown !!\n");

      server_log_event(string);
      return(ret);
    } else
      return(0);
}


/* This is the main server loop. We acquire the connection and select on the socket
 to be able to process incoming X10 events from the CM11A. When commands are
 received from the client, we process them */
void cycle_on_connections(int accept_socket)
{
  char line[MAXBUF];
  char *buffer;
  fd_set set_fds;
  struct timeval valtime;
  int select_result;

  /* acquire connection */
  sock = do_accept(accept_socket);

  /* no connection */
  if(!sock)
    return;

  /* connection: we cycle */
  while(1)
    {
      /* select on the socket */
      valtime.tv_sec = SERVER_TIME_OUT_SELECT_SECONDS;
      valtime.tv_usec = SERVER_TIME_OUT_SELECT_USECONDS;
      FD_ZERO(&set_fds);
      FD_SET(sock,&set_fds);
      select_result = select(sock+1,&set_fds, NULL, NULL, &valtime);

      /* no command received, we process X10 events if required */
      if(!select_result)
	{
	  x10_listen_to_events(x10_status,PERIOD_POLL_CM11A);
	  continue;
	}

      /* we received a command */
      if(!read(sock,line,MAXBUF))
	return;

      /* decoding commands and processing them */
      buffer = strtok((char *) line,"\n");
      buffer = strtok((char *) line," ");

      if(!strcmp(buffer,"send"))
	do_cmd_send();
      else
	if(!strcmp(buffer,"status"))
	  do_cmd_status();
	else
	  if(!strcmp(buffer,"status_all"))
	    do_cmd_status_all();
	  else
	    if(!strcmp(buffer,"log"))
	      do_cmd_log();
	    else
	      if(!strcmp(buffer,"listen"))
		do_cmd_listen();
	      else
		if(!strcmp(buffer,"nolisten"))
		  do_cmd_nolisten();
		else
		  if(!strcmp(buffer,"help"))
		    do_cmd_help();
		  else
		    error_cmd();
    }
}
